/**
  *  \file server/postobj.cc
  *  \brief Server: Implementation of "POST obj/..."
  */

#include <memory>
#include "server/postobj.h"
#include "server/jsonin.h"
#include "int/value.h"
#include "int/array.h"
#include "cpluslib/format.h"
#include "server/serverio.h"
#include "int/interror.h"
#include "cpluslib/ptrvector.h"
#include "server/objout.h"
#include "server/jsonout.h"
#include "util/misc.h"
#include "game/struct.h"
#include "server/objcmd.h"
#include "int/args.h"
#include "util/console.h"

ServerObjectCommandHandler*
createCommandHandler(string_t obj)
{
    int n;
    if (obj.size() > 8 && obj.compare(0, 8, "obj/ship", 8) == 0 && stringValue(obj.substr(8), n) && n > 0 && n <= NUM_SHIPS) {
        return new ServerShipCommandHandler(n);
    } else if (obj.size() > 10 && obj.compare(0, 10, "obj/planet", 10) == 0 && stringValue(obj.substr(10), n) && n > 0 && n <= NUM_PLANETS) {
        return new ServerPlanetCommandHandler(n);
    } else {
        return 0;
    }
}

void
doPostObject(ServerObjectCommandHandler* obj, const string_t& body)
{
    /* Parse request body */
    std::auto_ptr<IntValue> parsed;
    try {
        JsonTokenizer tok(body);
        tok.readNext();
        parsed.reset(readTree(tok));
        if (tok.getCurrent() != tok.kEOF) {
            writeClient("412 Syntax error (expecting end of line)");
            return;
        }
    }
    catch (IntError& e) {
        writeClient(format("412 Syntax error (%s)", e.what()));
        return;
    }

    /* Is it an array? */
    IntArray* a = dynamic_cast<IntArray*>(parsed.get());
    if (!a) {
        writeClient("412 Type error (not an array)");
        return;
    }

    /* Execute it */
    ptr_vector<ServerObjectWriter> result;
    Ptr<IntArrayData> aa = a->getData();
    for (uint32_t i = 0; i < aa->content.size(); ++i) {
        IntArray* cmd = dynamic_cast<IntArray*>(aa->content[i]);
        if (!cmd) {
            writeClient("412 Type error (command is not an array)");
            return;
        }
        Ptr<IntArrayData> cc = cmd->getData();
        IntStringValue* name = dynamic_cast<IntStringValue*>(cc->content[0]);
        if (!name) {
            writeClient("412 Type error (command is not a string)");
            return;
        }
        try {
            IntArgBlock args(cc->content, 1, cc->content.size() - 1);
            obj->processCommand(name->getValue(), args, result);
        }
        catch (IntError& e) {
            console.write(LOG_ERROR, format("Execution of command '%s' failed: %s", name->getValue(), e.what()));
            writeClient(format("450 Failed (%s)", e.what()));
            return;
        }
    }

    /* Generate result */
    JsonWriter w;
    writeClient("200 OK");
    w.start();
    for (ptr_vector<ServerObjectWriter>::iterator i = result.begin(); i != result.end(); ++i) {
        bool did = false;
        for (ptr_vector<ServerObjectWriter>::iterator j = result.begin(); j != i; ++j) {
            if ((*i)->isSame(*j)) {
                did = true;
                break;
            }
        }
        if (!did) {
            (*i)->write(w);
        }
    }
    w.end();
    writeClientTerminator();
}
